"""
Copyright 2021-2023 MICRORISC s.r.o.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

	http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""
import argparse
import os
import sys
import sentry_sdk

from sentry_sdk.integrations.atexit import AtexitIntegration
from Setter import __version__
from Setter.Common.AuxiliaryFunctions import AuxiliaryFunctions
from Setter.Common.Command import Command
from Setter.Common.GatewayFile import GatewayFile
from Setter.Enums.ErrorCodes import ErrorCodes
from Setter.Enums.Software import FactoryResetSoftware, SystemdServices
from Setter.Exceptions.BackupException import BackupException
from Setter.Exceptions.SetterBaseException import SetterBaseException
from Setter.ControllerConfig import ControllerConfig
from Setter.DaemonConfig import DaemonConfig
from Setter.FactoryReset import FactoryReset
from Setter.GatewayConfig import GatewayConfig
from Setter.Logging import Logging
from Setter.UploaderConfig import UploaderConfig
from Setter.WebappConfig import WebappConfig


def main():
	"""Setter entry point."""
	parser = argparse.ArgumentParser(prog='IQRF Gateway Setter', description='Configuration tool for IQRF Gateway.')
	exclusive = parser.add_mutually_exclusive_group()
	exclusive.add_argument(
		'-c', '--configure', dest='configure_sw', action='store', nargs='+',
		choices=[
			'gateway',
			'controller',
			'daemon',
			'uploader',
			'webapp'
		],
		help='Configure selected software, tools and utilities.'
	)
	exclusive.add_argument(
		'-r', '--factory-reset', dest='factory_reset_sw', action='store', nargs='+',
		choices=[
			FactoryResetSoftware.ALL.value,
			FactoryResetSoftware.CLOUD_PROVISIONING.value,
			FactoryResetSoftware.DAEMON.value,
			FactoryResetSoftware.IQAROS.value,
			FactoryResetSoftware.NETWORK_MANAGER.value,
			FactoryResetSoftware.WEBAPP.value
		],
		help='Factory reset configuration of selected software.'
	)
	parser.add_argument(
		'-f', '--force', dest='force', action='store_true',
		help='Force re-configure IQRF Gateway device.'
	)
	parser.add_argument(
		'-R', '--restart', dest='restart', action='store_true',
		help='Service is restarted after configuration. Has no effect unless Controller or Daemon option is used.'
	)
	parser.add_argument(
		'-V', '--verbosity', dest='verbosity', action='store', required=False, default='INFO', type=str,
		choices=['DEBUG', 'INFO', 'WARNING', 'ERROR', 'CRITICAL'], help='Sets logger verbosity'
	)
	parser.add_argument('-v', '--version', action='version', version='%(prog)s v' + __version__)

	args = parser.parse_args()

	sentry_sdk.init(
		"https://96cafcb5cf7644e38daf445dc0b862c4@sentry.iqrf.org/7",
		traces_sample_rate=1.0,
		integrations=[
			AtexitIntegration(AuxiliaryFunctions.atexit_callback())
		]
	)

	logger = Logging.get_logger()
	Logging.set_level(args.verbosity)
	logger.info('IQRF Gateway Setter v%s', __version__)

	if args.factory_reset_sw is not None:
		try:
			FactoryReset.perform_reset(args.factory_reset_sw, FactoryResetSoftware.ALL in args.factory_reset_sw)
		except SetterBaseException as setter_error:
			return setter_error.code
	elif args.configure_sw is not None:
		setter_flag = os.path.exists('/var/cache/iqrf-gateway-setter')
		gateway_force = 'gateway' in args.configure_sw and (not setter_flag or args.force)
		try:
			# read gateway config
			gateway_config = GatewayFile.read()

			# get image version
			GatewayConfig.get_image_version(gateway_config['gwImage'])

			# configure controller
			if 'controller' in args.configure_sw or gateway_force:
				try:
					ControllerConfig.backup()
					ControllerConfig.configure(gateway_config['gwImage'], gateway_config['gwProduct'])
					if args.restart:
						Command.restart_service(SystemdServices.CONTROLLER)
				except BackupException as backup_error:
					raise backup_error
				except SetterBaseException as controller_error:
					ControllerConfig.restore()
					raise controller_error

			# configure daemon
			if 'daemon' in args.configure_sw or gateway_force:
				try:
					DaemonConfig.backup()
					DaemonConfig.configure_interface(gateway_config['gwProduct'], gateway_config['gwInterface'])
					if args.restart:
						Command.restart_service(SystemdServices.DAEMON)
				except BackupException as backup_error:
					raise backup_error
				except SetterBaseException as daemon_error:
					DaemonConfig.restore()
					raise daemon_error

			# configure uploader
			if 'uploader' in args.configure_sw or gateway_force:
				try:
					UploaderConfig.backup()
					UploaderConfig.configure(gateway_config['gwProduct'])
				except BackupException as backup_error:
					raise backup_error
				except SetterBaseException as uploader_error:
					UploaderConfig.restore()
					raise uploader_error

			# configure webapp
			if 'webapp' in args.configure_sw:
				try:
					WebappConfig.configure(gateway_config)
				except SetterBaseException as webapp_error:
					raise webapp_error

			# configure gateway
			if 'gateway' in args.configure_sw:
				if gateway_force:
					GatewayConfig.configure(gateway_config)
				else:
					logger.info('IQRF Gateway device already configured.')
		except SetterBaseException as setter_error:
			return setter_error.code

	return ErrorCodes.OK


if __name__ == '__main__':
	sys.exit(main())
